# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


class PostFinanceCheckoutSdkException(Exception):
    """
    Exception thrown when an internal SDK error occurs
    """
    def __init__(self, code: str, message: str):
        """
        Constructor.

        :param code: SDK error code (string)
        :param message: Exception message details
        """
        self.code = code
        self.message = message
        super().__init__(f"Error code: {code}. {message}")

    def __str__(self):
        return super().__str__()


# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import StrictStr

from postfinancecheckout.api_client import ApiClient, RequestSerialized
from postfinancecheckout.api_response import ApiResponse
from postfinancecheckout.rest import RESTResponseType

from postfinancecheckout.utils.encryption_util import EncryptionUtil
from postfinancecheckout.postfinancecheckout_sdk_exception import PostFinanceCheckoutSdkException
from postfinancecheckout.sdk_exception_error_codes import SdkExceptionErrorCodes
import re


class WebhookEncryptionKeysService:
    def __init__(self, configuration):
        self.api_client = ApiClient(configuration=configuration)


    @validate_call
    def get_webhooks_encryption_keys_id(
        self,
        id: StrictStr,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> str:
        """Retrieve a webhook encryption key



        :param id: (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_webhooks_encryption_keys_id_serialize(
            id=id,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "str",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_webhooks_encryption_keys_id_with_http_info(
        self,
        id: StrictStr,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[str]:
        """Retrieve a webhook encryption key



        :param id: (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_webhooks_encryption_keys_id_serialize(
            id=id,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "str",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_webhooks_encryption_keys_id_without_preload_content(
        self,
        id: StrictStr,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve a webhook encryption key



        :param id: (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_webhooks_encryption_keys_id_serialize(
            id=id,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "str",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_webhooks_encryption_keys_id_serialize(
        self,
        id,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/webhooks/encryption-keys/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )

    def is_content_valid(self, signature_header, content_to_verify):
        """Verify webhook content using signature header

        Verifies webhook content using signature header.

        :param str signature_header: Signature header 'X-Signature' value from the Http request
        :param str content_to_verify: Raw webhook content in String format
        :return: bool if webhook content body conforms with signature header
        """
        # Regular expression to extract algorithm, keyId, and signature from the signature_header
        regex = r"^algorithm=([a-zA-Z0-9]+),\skeyId=([a-z0-9\-]+),\s{1}signature=([a-zA-Z0-9+\/=]+)$"
        pattern = re.compile(regex)
        matcher = pattern.match(signature_header)

        if matcher:
            signature_algorithm = matcher.group(1)
            public_key_id = matcher.group(2)
            content_signature = matcher.group(3)

            public_key_response = WebhookEncryptionKeysService.get_webhooks_encryption_keys_id(self, public_key_id)

            if not public_key_response:
                raise PostFinanceCheckoutSdkException(
                    SdkExceptionErrorCodes.UNKNOWN_WEBHOOK_ENCRYPTION_PUBLIC_KEY.code,
                    f"Could not find public key with id {public_key_id}"
                )

            return EncryptionUtil.is_content_valid(content_to_verify, content_signature, public_key_response,
                                                   signature_algorithm)
        else:
            raise PostFinanceCheckoutSdkException(
                SdkExceptionErrorCodes.INVALID_WEBHOOK_ENCRYPTION_HEADER_FORMAT.code,
                "Invalid webhook signature header. Expected format: 'algorithm=<algorithm>, keyId=<keyId>, signature=<signature>'"
            )





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from postfinancecheckout.models.product_fee_type import ProductFeeType
from postfinancecheckout.models.subscription_ledger_entry_state import SubscriptionLedgerEntryState
from postfinancecheckout.models.tax import Tax
from typing import Optional, Set
from typing_extensions import Self

class SubscriptionLedgerEntry(BaseModel):
    """
    The subscription ledger entry represents a single change on the subscription balance.
    """ # noqa: E501
    quantity: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The number of items that were consumed.")
    amount_excluding_tax: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The leger entry's amount with discounts applied, excluding taxes.", alias="amountExcludingTax")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    subscription_version: Optional[StrictInt] = Field(default=None, description="The subscription version that the ledger entry belongs to.", alias="subscriptionVersion")
    external_id: Optional[StrictStr] = Field(default=None, description="A client-generated nonce which uniquely identifies some action to be executed. Subsequent requests with the same external ID do not execute the action again, but return the original result.", alias="externalId")
    taxes: Optional[List[Tax]] = Field(default=None, description="A set of tax lines, each of which specifies a tax applied to the ledger entry.")
    fee_type: Optional[ProductFeeType] = Field(default=None, alias="feeType")
    title: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=150)]] = Field(default=None, description="The title that indicates what the ledger entry is about.")
    created_on: Optional[datetime] = Field(default=None, description="The date and time when the object was created.", alias="createdOn")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    component_reference_name: Optional[StrictStr] = Field(default=None, alias="componentReferenceName")
    subscription_metric_id: Optional[StrictInt] = Field(default=None, alias="subscriptionMetricId")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    pro_rata_calculated: Optional[StrictBool] = Field(default=None, alias="proRataCalculated")
    created_by: Optional[StrictInt] = Field(default=None, description="The ID of the user the ledger entry was created by.", alias="createdBy")
    component_reference_sku: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, alias="componentReferenceSku")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[SubscriptionLedgerEntryState] = None
    amount_including_tax: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The leger entry's amount with discounts applied, including taxes.", alias="amountIncludingTax")
    discount_including_tax: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The discount allocated to the ledger entry, including taxes.", alias="discountIncludingTax")
    tax_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The sum of all taxes applied to the ledger entry.", alias="taxAmount")
    aggregated_tax_rate: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total tax rate applied to the ledger entry, calculated from the rates of all tax lines.", alias="aggregatedTaxRate")
    __properties: ClassVar[List[str]] = ["quantity", "amountExcludingTax", "plannedPurgeDate", "subscriptionVersion", "externalId", "taxes", "feeType", "title", "createdOn", "version", "componentReferenceName", "subscriptionMetricId", "linkedSpaceId", "proRataCalculated", "createdBy", "componentReferenceSku", "id", "state", "amountIncludingTax", "discountIncludingTax", "taxAmount", "aggregatedTaxRate"]

    @field_validator('component_reference_sku')
    def component_reference_sku_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"([0-9a-zA-Z\-_]+)", value):
            raise ValueError(r"must validate the regular expression /([0-9a-zA-Z\-_]+)/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SubscriptionLedgerEntry from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "quantity",
            "amount_excluding_tax",
            "planned_purge_date",
            "subscription_version",
            "external_id",
            "taxes",
            "title",
            "created_on",
            "version",
            "component_reference_name",
            "subscription_metric_id",
            "linked_space_id",
            "pro_rata_calculated",
            "created_by",
            "component_reference_sku",
            "id",
            "amount_including_tax",
            "discount_including_tax",
            "tax_amount",
            "aggregated_tax_rate",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in taxes (list)
        _items = []
        if self.taxes:
            for _item_taxes in self.taxes:
                if _item_taxes:
                    _items.append(_item_taxes.to_dict())
            _dict['taxes'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SubscriptionLedgerEntry from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "quantity": obj.get("quantity"),
            "amountExcludingTax": obj.get("amountExcludingTax"),
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "subscriptionVersion": obj.get("subscriptionVersion"),
            "externalId": obj.get("externalId"),
            "taxes": [Tax.from_dict(_item) for _item in obj["taxes"]] if obj.get("taxes") is not None else None,
            "feeType": obj.get("feeType"),
            "title": obj.get("title"),
            "createdOn": obj.get("createdOn"),
            "version": obj.get("version"),
            "componentReferenceName": obj.get("componentReferenceName"),
            "subscriptionMetricId": obj.get("subscriptionMetricId"),
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "proRataCalculated": obj.get("proRataCalculated"),
            "createdBy": obj.get("createdBy"),
            "componentReferenceSku": obj.get("componentReferenceSku"),
            "id": obj.get("id"),
            "state": obj.get("state"),
            "amountIncludingTax": obj.get("amountIncludingTax"),
            "discountIncludingTax": obj.get("discountIncludingTax"),
            "taxAmount": obj.get("taxAmount"),
            "aggregatedTaxRate": obj.get("aggregatedTaxRate")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from enum import Enum

class SdkExceptionErrorCodes(Enum):
    """
    SDK Local Error Codes
    """
    # Unknown webhook signature public key
    UNKNOWN_WEBHOOK_ENCRYPTION_PUBLIC_KEY = "unknown_public_key"
    # General webhook encryption error
    WEBHOOK_ENCRYPTION_GENERAL_ERROR = "encryption_error"
    # Invalid webhook signature public key
    INVALID_WEBHOOK_ENCRYPTION_PUBLIC_KEY = "invalid_public_key"
    # Invalid webhook signature header
    INVALID_WEBHOOK_ENCRYPTION_HEADER_FORMAT = "invalid_webhook_header"
    # Unsupported webhook signature algorithm
    UNSUPPORTED_WEBHOOK_ENCRYPTION_ALGORYTHM = "unsupported_encryption_algorythm"
    # Unknown webhook encryption provider
    UNKNOWN_WEBHOOK_ENCRYPTION_PROVIDER = "unknown_encryption_provider"
    # Encryption verifier initialization error
    WEBHOOK_ENCRYPTION_VERIFIER_INIT_ERROR = "verifier_init_failure"
    # Error during content update in encryption verifier
    WEBHOOK_ENCRYPTION_VERIFIER_CONTENT_UPDATE_ERROR = "content_update_failure"
    # Encryption signature verification failed
    WEBHOOK_ENCRYPTION_SIGNATURE_VERIFICATION_FAILED = "signature_verification_failure"
    # Invalid webhook content signature
    INVALID_WEBHOOK_ENCRYPTION_CONTENT_SIGNATURE = "invalid_content_signature"
    # Missing webhook signature algorithm value
    MISSING_WEBHOOK_ENCRYPTION_ALGORYTHM = "missing_encryption_algorythm"

    def __init__(self, code):
        self._code = code

    @property
    def code(self):
        return self._code

    def matches(self, sdk_exception):
        """
        Checks if the given exception matches this error code.
        """
        return sdk_exception.code == self.code


# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from postfinancecheckout.models.payment_terminal_dcc_transaction_sum import PaymentTerminalDccTransactionSum
from postfinancecheckout.models.payment_terminal_transaction_sum import PaymentTerminalTransactionSum
from typing import Optional, Set
from typing_extensions import Self

class PaymentTerminalTransactionSummary(BaseModel):
    """
    PaymentTerminalTransactionSummary
    """ # noqa: E501
    reference: Optional[StrictInt] = Field(default=None, description="The unique reference assigned to this transaction summary.")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    transaction_sums: Optional[List[PaymentTerminalTransactionSum]] = Field(default=None, description="The total monetary amounts of all transactions, organized and grouped by brand and currency.", alias="transactionSums")
    dcc_transaction_sums: Optional[List[PaymentTerminalDccTransactionSum]] = Field(default=None, description="Detailed breakdown of Dynamic Currency Conversion (DCC) transactions, showing transaction amounts in both original and converted currencies, grouped by payment brand.", alias="dccTransactionSums")
    ended_on: Optional[datetime] = Field(default=None, description="The end of the time period covered by this summary report.", alias="endedOn")
    balance_amount_per_currency: Optional[Dict[str, Union[StrictFloat, StrictInt]]] = Field(default=None, description="The overall transaction volume in each processed currency.", alias="balanceAmountPerCurrency")
    payment_terminal: Optional[StrictInt] = Field(default=None, description="The payment terminal that processed the transactions included in this summary report.", alias="paymentTerminal")
    receipt: Optional[StrictStr] = Field(default=None, description="The HTML content of the transaction summary receipt.")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    number_of_transactions: Optional[StrictInt] = Field(default=None, description="The total count of all transactions processed by the terminal during the summary period.", alias="numberOfTransactions")
    started_on: Optional[datetime] = Field(default=None, description="The beginning of the time period covered by this summary report.", alias="startedOn")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    __properties: ClassVar[List[str]] = ["reference", "linkedSpaceId", "transactionSums", "dccTransactionSums", "endedOn", "balanceAmountPerCurrency", "paymentTerminal", "receipt", "id", "numberOfTransactions", "startedOn", "version"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PaymentTerminalTransactionSummary from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "reference",
            "linked_space_id",
            "transaction_sums",
            "dcc_transaction_sums",
            "ended_on",
            "balance_amount_per_currency",
            "payment_terminal",
            "receipt",
            "id",
            "number_of_transactions",
            "started_on",
            "version",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in transaction_sums (list)
        _items = []
        if self.transaction_sums:
            for _item_transaction_sums in self.transaction_sums:
                if _item_transaction_sums:
                    _items.append(_item_transaction_sums.to_dict())
            _dict['transactionSums'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in dcc_transaction_sums (list)
        _items = []
        if self.dcc_transaction_sums:
            for _item_dcc_transaction_sums in self.dcc_transaction_sums:
                if _item_dcc_transaction_sums:
                    _items.append(_item_dcc_transaction_sums.to_dict())
            _dict['dccTransactionSums'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PaymentTerminalTransactionSummary from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "reference": obj.get("reference"),
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "transactionSums": [PaymentTerminalTransactionSum.from_dict(_item) for _item in obj["transactionSums"]] if obj.get("transactionSums") is not None else None,
            "dccTransactionSums": [PaymentTerminalDccTransactionSum.from_dict(_item) for _item in obj["dccTransactionSums"]] if obj.get("dccTransactionSums") is not None else None,
            "endedOn": obj.get("endedOn"),
            "balanceAmountPerCurrency": obj.get("balanceAmountPerCurrency"),
            "paymentTerminal": obj.get("paymentTerminal"),
            "receipt": obj.get("receipt"),
            "id": obj.get("id"),
            "numberOfTransactions": obj.get("numberOfTransactions"),
            "startedOn": obj.get("startedOn"),
            "version": obj.get("version")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictInt, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from postfinancecheckout.models.web_app_confirmation_response import WebAppConfirmationResponse

from postfinancecheckout.api_client import ApiClient, RequestSerialized
from postfinancecheckout.api_response import ApiResponse
from postfinancecheckout.rest import RESTResponseType



class WebAppsService:
    def __init__(self, configuration):
        self.api_client = ApiClient(configuration=configuration)


    @validate_call
    def get_web_apps_installed(
        self,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> bool:
        """Check whether the web app is installed

        Return true if the web app is installed in the provided space.
        

        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_web_apps_installed_serialize(
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "bool",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_web_apps_installed_with_http_info(
        self,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[bool]:
        """Check whether the web app is installed

        Return true if the web app is installed in the provided space.
        

        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_web_apps_installed_serialize(
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "bool",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_web_apps_installed_without_preload_content(
        self,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Check whether the web app is installed

        Return true if the web app is installed in the provided space.
        

        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_web_apps_installed_serialize(
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "bool",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_web_apps_installed_serialize(
        self,
        space,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        if space is not None:
            _header_params['Space'] = space
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/web-apps/installed',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def post_web_apps_confirm_code(
        self,
        code: Annotated[StrictStr, Field(description="The activation code passed to the web app after permissions were granted.")],
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> WebAppConfirmationResponse:
        """Confirm a web app installation

        Confirm the installation of a web app. This has to be done when the user returns to the web app after granting permissions, using the activation code provided in the request.
        

        :param code: The activation code passed to the web app after permissions were granted. (required)
        :type code: str
        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_web_apps_confirm_code_serialize(
            code=code,
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WebAppConfirmationResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def post_web_apps_confirm_code_with_http_info(
        self,
        code: Annotated[StrictStr, Field(description="The activation code passed to the web app after permissions were granted.")],
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[WebAppConfirmationResponse]:
        """Confirm a web app installation

        Confirm the installation of a web app. This has to be done when the user returns to the web app after granting permissions, using the activation code provided in the request.
        

        :param code: The activation code passed to the web app after permissions were granted. (required)
        :type code: str
        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_web_apps_confirm_code_serialize(
            code=code,
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WebAppConfirmationResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def post_web_apps_confirm_code_without_preload_content(
        self,
        code: Annotated[StrictStr, Field(description="The activation code passed to the web app after permissions were granted.")],
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Confirm a web app installation

        Confirm the installation of a web app. This has to be done when the user returns to the web app after granting permissions, using the activation code provided in the request.
        

        :param code: The activation code passed to the web app after permissions were granted. (required)
        :type code: str
        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_web_apps_confirm_code_serialize(
            code=code,
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "WebAppConfirmationResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _post_web_apps_confirm_code_serialize(
        self,
        code,
        expand,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'expand': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if code is not None:
            _path_params['code'] = code
        # process the query parameters
        if expand is not None:
            
            _query_params.append(('expand', expand))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='POST',
            resource_path='/web-apps/confirm/{code}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def post_web_apps_uninstall(
        self,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Uninstall a web app

        Uninstall the web app from the provided space.
        

        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_web_apps_uninstall_serialize(
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def post_web_apps_uninstall_with_http_info(
        self,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Uninstall a web app

        Uninstall the web app from the provided space.
        

        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_web_apps_uninstall_serialize(
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def post_web_apps_uninstall_without_preload_content(
        self,
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Uninstall a web app

        Uninstall the web app from the provided space.
        

        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_web_apps_uninstall_serialize(
            space=space,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '409': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _post_web_apps_uninstall_serialize(
        self,
        space,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        if space is not None:
            _header_params['Space'] = space
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='POST',
            resource_path='/web-apps/uninstall',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )






# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from postfinancecheckout.models.billing_cycle_model import BillingCycleModel
from postfinancecheckout.models.subscription_product import SubscriptionProduct
from postfinancecheckout.models.subscription_product_version_state import SubscriptionProductVersionState
from postfinancecheckout.models.tax_calculation import TaxCalculation
from typing import Optional, Set
from typing_extensions import Self

class SubscriptionProductVersion(BaseModel):
    """
    SubscriptionProductVersion
    """ # noqa: E501
    retiring_finished_on: Optional[datetime] = Field(default=None, description="The date and time when the product version was retired.", alias="retiringFinishedOn")
    enabled_currencies: Optional[List[StrictStr]] = Field(default=None, description="The three-letter codes (ISO 4217 format) of the currencies that the product version supports.", alias="enabledCurrencies")
    product: Optional[SubscriptionProduct] = None
    retiring_started_on: Optional[datetime] = Field(default=None, description="The date and time when the product version's retirement was started.", alias="retiringStartedOn")
    tax_calculation: Optional[TaxCalculation] = Field(default=None, alias="taxCalculation")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    created_on: Optional[datetime] = Field(default=None, description="The date and time when the product version was created.", alias="createdOn")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    reference: Optional[Annotated[str, Field(strict=True, max_length=125)]] = Field(default=None, description="The reference used to identify the product version.")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    activated_on: Optional[datetime] = Field(default=None, description="The date and time when the product version was activated.", alias="activatedOn")
    billing_cycle: Optional[StrictStr] = Field(default=None, description="The recurring period of time, typically monthly or annually, for which a subscriber is charged.", alias="billingCycle")
    default_currency: Optional[StrictStr] = Field(default=None, description="The three-letter code (ISO 4217 format) of the product version's default currency.", alias="defaultCurrency")
    name: Optional[Dict[str, StrictStr]] = Field(default=None, description="The localized name of the product that is displayed to the customer.")
    minimal_number_of_periods: Optional[StrictInt] = Field(default=None, description="The minimum number of periods the subscription will run before it can be terminated.", alias="minimalNumberOfPeriods")
    obsoleted_on: Optional[datetime] = Field(default=None, description="The date and time when the product version was made obsolete.", alias="obsoletedOn")
    billing_cycle_model: Optional[BillingCycleModel] = Field(default=None, alias="billingCycleModel")
    comment: Optional[StrictStr] = Field(default=None, description="A comment that describes the product version and why it was created. It is not disclosed to the subscriber.")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    increment_number: Optional[StrictInt] = Field(default=None, description="Whenever a new version of a product is created, the number is increased and assigned.", alias="incrementNumber")
    state: Optional[SubscriptionProductVersionState] = None
    number_of_notice_periods: Optional[StrictInt] = Field(default=None, description="The number of periods the subscription will keep running after its termination was requested.", alias="numberOfNoticePeriods")
    __properties: ClassVar[List[str]] = ["retiringFinishedOn", "enabledCurrencies", "product", "retiringStartedOn", "taxCalculation", "plannedPurgeDate", "createdOn", "version", "reference", "linkedSpaceId", "activatedOn", "billingCycle", "defaultCurrency", "name", "minimalNumberOfPeriods", "obsoletedOn", "billingCycleModel", "comment", "id", "incrementNumber", "state", "numberOfNoticePeriods"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SubscriptionProductVersion from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "retiring_finished_on",
            "enabled_currencies",
            "retiring_started_on",
            "planned_purge_date",
            "created_on",
            "version",
            "reference",
            "linked_space_id",
            "activated_on",
            "billing_cycle",
            "default_currency",
            "name",
            "minimal_number_of_periods",
            "obsoleted_on",
            "comment",
            "id",
            "increment_number",
            "number_of_notice_periods",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of product
        if self.product:
            _dict['product'] = self.product.to_dict()
        # override the default output from pydantic by calling `to_dict()` of billing_cycle_model
        if self.billing_cycle_model:
            _dict['billingCycleModel'] = self.billing_cycle_model.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SubscriptionProductVersion from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "retiringFinishedOn": obj.get("retiringFinishedOn"),
            "enabledCurrencies": obj.get("enabledCurrencies"),
            "product": SubscriptionProduct.from_dict(obj["product"]) if obj.get("product") is not None else None,
            "retiringStartedOn": obj.get("retiringStartedOn"),
            "taxCalculation": obj.get("taxCalculation"),
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "createdOn": obj.get("createdOn"),
            "version": obj.get("version"),
            "reference": obj.get("reference"),
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "activatedOn": obj.get("activatedOn"),
            "billingCycle": obj.get("billingCycle"),
            "defaultCurrency": obj.get("defaultCurrency"),
            "name": obj.get("name"),
            "minimalNumberOfPeriods": obj.get("minimalNumberOfPeriods"),
            "obsoletedOn": obj.get("obsoletedOn"),
            "billingCycleModel": BillingCycleModel.from_dict(obj["billingCycleModel"]) if obj.get("billingCycleModel") is not None else None,
            "comment": obj.get("comment"),
            "id": obj.get("id"),
            "incrementNumber": obj.get("incrementNumber"),
            "state": obj.get("state"),
            "numberOfNoticePeriods": obj.get("numberOfNoticePeriods")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import unittest
import uuid

from constants import API_CONFIG, SPACE_ID, MOCK_CARD_DATA, TEST_CUSTOMER_ID, TEST_CARD_PAYMENT_METHOD_CONFIGURATION_ID
from utils import Utils
from postfinancecheckout.service import TransactionsService, TokensService
from postfinancecheckout.models import ( TransactionState,
                            TransactionCompletionState,
                            TokenizationMode,
                            CustomersPresence,
                            TransactionVoidState,
                            TransactionCompletionBehavior,
                            TransactionCompletionDetails,
                            TransactionPending,
                            TokenCreate,
                            CreationEntityState,
                            TokenUpdate,
                            TokenizedCardDataCreate,
                            TokenizedCardRequest,
                            TerminalReceiptFormat,
                            LineItemCreate)
from postfinancecheckout import ApiException

class TestTransactionsService(unittest.TestCase):
    """postfinancecheckout transactions service tests"""

    def setUp(self):
        self.tokens_service = TokensService(API_CONFIG)
        self.transactions_service = TransactionsService(API_CONFIG)
        self.INTEGRATION_MODE = "payment_page"


    def test_create_and_find_pending_transaction(self):
        """
        Creates a new transaction.

        Verifies that:
        - Transaction state is PENDING
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        found = self.transactions_service.get_payment_transactions_id(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            TransactionState.PENDING,
            transaction.state,
            "Transaction state must be PENDING"
        )
        self.assertEqual(
            transaction.merchant_reference,
            found.merchant_reference,
            "Merchant reference should match."
        )

    def test_confirm_should_make_transaction_confirmed(self):
        """
        Confirms a pending transaction.

        Verifies that:
        - Transaction state is CONFIRMED
        - Transaction entity version is correctly incremented
        - Merchant reference is correctly updated
        """

        transaction_create = Utils.get_transaction_create_payload()
        transaction_create.merchant_reference = "Test Initial Confirm"
        created = self.create(transaction_create)

        transaction_pending = TransactionPending(version= 2)
        transaction_pending.merchant_reference = "Test Confirm"

        confirmed = self.transactions_service.post_payment_transactions_id_confirm(
            created.id, SPACE_ID, transaction_pending
        )

        self.assertEqual(
            TransactionState.CONFIRMED,
            confirmed.state,
            "Transaction state must be CONFIRMED"
        )
        self.assertEqual(3, confirmed.version, "Version should match.")
        self.assertEqual(
            transaction_pending.merchant_reference,
            confirmed.merchant_reference,
            "Merchant reference should match."
        )
    def test_process_deferred_transaction_should_make_transaction_authorized(self):
        """
        Processes a deferred transaction.

        Verifies that:
        - Transaction state is AUTHORIZED
        """

        transaction_create = Utils.get_transaction_create_payload()
        transaction_create.tokenization_mode = TokenizationMode.FORCE_CREATION
        transaction_create.customers_presence = CustomersPresence.NOT_PRESENT
        transaction_create.completion_behavior = TransactionCompletionBehavior.COMPLETE_DEFERRED

        transaction = self.create(transaction_create)

        processed = self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        self.assertEqual(
            TransactionState.AUTHORIZED,
            processed.state,
            "Transaction state must be AUTHORIZED"
        )

    def test_process_via_charge_flow_should_make_transaction_processing(self):
        """
        Processes a transaction via charge flow.

        Verifies that:
        - Transaction state is PROCESSING
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        processing_transaction = self.transactions_service.post_payment_transactions_id_charge_flow_apply(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            TransactionState.PROCESSING,
            processing_transaction.state,
            "Transaction state must be PROCESSING"
        )

    def test_cancel_charge_flow_should_make_transaction_failed(self):
        """
        Processes and cancels a transaction via charge flow.

        Verifies that:
        - Initially, transaction state is PROCESSING
        - After cancellation, transaction state is FAILED
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        processing_transaction = self.transactions_service.post_payment_transactions_id_charge_flow_apply(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            TransactionState.PROCESSING,
            processing_transaction.state,
            "Transaction state must be PROCESSING"
        )

        failed_transaction = self.transactions_service.post_payment_transactions_id_charge_flow_cancel(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            TransactionState.FAILED,
            failed_transaction.state,
            "Transaction state must be FAILED"
        )

    def test_fetch_payment_page_url_should_return_valid_url(self):
        """
        Processes a transaction and retrieves payment page URL.

        Verifies that:
        - Retrieved URL contains space ID
        - Retrieved URL contains transaction ID
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        processing_transaction = self.transactions_service.post_payment_transactions_id_charge_flow_apply(
            transaction.id, SPACE_ID
        )

        url = self.transactions_service.get_payment_transactions_id_payment_page_url(
            transaction.id, SPACE_ID
        )

        self.assertIn(f"/s/{SPACE_ID}", url, "Space id should be present in url")
        self.assertIn("securityToken=", url, "Security token should be present in url")
        self.assertIn(str(processing_transaction.id), url, "Transaction id should be present in url")

    def test_fetch_charge_flow_url_should_return_valid_url(self):
        """
        Processes a transaction via charge flow and retrieves payment page URL.

        Verifies that:
        - Transaction state is PROCESSING
        - Retrieved URL contains space ID
        - Retrieved URL contains transaction ID
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        processing_transaction = self.transactions_service.post_payment_transactions_id_charge_flow_apply(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            TransactionState.PROCESSING,
            processing_transaction.state,
            "Transaction state must be PROCESSING"
        )

        url = self.transactions_service.get_payment_transactions_id_charge_flow_payment_page_url(
            processing_transaction.id, SPACE_ID
        )

        self.assertIn(str(SPACE_ID), url, "Url must contain space id")
        self.assertIn(str(processing_transaction.id), url, "Url must contain transaction id")
        self.assertIn("securityToken=", url, "Url must contain security token")

    def test_complete_online_should_make_transaction_completion_state_successful(self):
        """
        Authorizes and completes a transaction online using card details.

        Verifies that:
        - Transaction completion state is SUCCESSFUL
        - Transaction state is FULFILL
        """

        transaction_create = Utils.get_transaction_create_payload()
        transaction_create.tokenization_mode = TokenizationMode.FORCE_CREATION
        transaction_create.customers_presence = CustomersPresence.NOT_PRESENT
        transaction_create.completion_behavior = TransactionCompletionBehavior.COMPLETE_IMMEDIATELY

        transaction = self.create(transaction_create)

        authorized_transaction = self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        processed_transaction = self.transactions_service.post_payment_transactions_id_complete_online(
            authorized_transaction.id, SPACE_ID
        )

        self.assertEqual(
            TransactionCompletionState.SUCCESSFUL,
            processed_transaction.state,
            "Transaction completion state must be SUCCESSFUL"
        )

        completed_transaction = self.transactions_service.get_payment_transactions_id(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            TransactionState.FULFILL,
            completed_transaction.state,
            "Transaction state must be FULFILLED"
        )

    def test_complete_online_partially_should_make_transaction_completion_state_successful(self):
        """
        Authorizes and completes a transaction online partially using card details.

        Verifies that:
        - Transaction completion state is SUCCESSFUL
        - Transaction state is FULFILL
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        authorized_transaction = self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        tcd = TransactionCompletionDetails()
        tcd.external_id = str(uuid.uuid4())

        processed_transaction = self.transactions_service.post_payment_transactions_id_complete_partially_online(
            authorized_transaction.id, SPACE_ID, tcd
        )

        self.assertEqual(
            TransactionCompletionState.SUCCESSFUL,
            processed_transaction.state,
            "Transaction completion state must be SUCCESSFUL"
        )

        completed_transaction = self.transactions_service.get_payment_transactions_id(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            TransactionState.FULFILL,
            completed_transaction.state,
            "Transaction state must be FULFILLED"
        )



    def test_void_transaction_online_should_return_voided_transaction(self):
        """
        Authorizes and voids a transaction online.

        Verifies that:
        - Transaction void state is SUCCESSFUL
        - Transaction state is VOIDED
        """

        transaction_create = Utils.get_transaction_create_payload()
        transaction_create.tokenization_mode = TokenizationMode.FORCE_CREATION
        transaction_create.customers_presence = CustomersPresence.NOT_PRESENT
        transaction_create.completion_behavior = TransactionCompletionBehavior.COMPLETE_DEFERRED

        transaction = self.create(transaction_create)

        authorized_transaction = self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        self.assertEqual(
            TransactionState.AUTHORIZED,
            authorized_transaction.state,
            "Transaction state should be AUTHORIZED"
        )

        expand = {"transaction"}

        transaction_void = self.transactions_service.post_payment_transactions_id_void_online(
            authorized_transaction.id, SPACE_ID, expand
        )

        self.assertEqual(
            TransactionVoidState.SUCCESSFUL,
            transaction_void.state,
            "Transaction void state should be SUCCESSFUL"
        )

        self.assertIsNotNone(transaction_void.transaction)

        self.assertEqual(
            TransactionState.VOIDED,
            transaction_void.transaction.state,
            "Transaction state should be VOIDED"
        )


    def test_check_if_possible_to_create_token_for_fulfilled_transaction(self):
        """
        Creates, authorizes and completes a transaction.

        Verifies that:
        - It's possible to create a transaction token for the fulfilled transaction
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        authorized_transaction = self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        fulfilled_transaction_completion = self.transactions_service.post_payment_transactions_id_complete_online(
            authorized_transaction.id, SPACE_ID
        )

        fulfilled_transaction = self.transactions_service.get_payment_transactions_id(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            TransactionState.FULFILL,
            fulfilled_transaction.state,
            "Transaction state must be FULFILL"
        )

        self.assertEqual(
            TransactionCompletionState.SUCCESSFUL,
            fulfilled_transaction_completion.state,
            "Transaction completion state must be SUCCESSFUL"
        )

        self.assertTrue(
            self.transactions_service.get_payment_transactions_id_check_token_creation_possible(
                fulfilled_transaction.id, SPACE_ID
            ),
            "Should be possible to create token for successful transaction"
        )

    def test_check_if_possible_to_create_token_for_voided_transaction(self):
        """
        Creates, authorizes and voids a transaction.

        Verifies that:
        - It's NOT possible to create a transaction token for the voided transaction
        """

        transaction_create = Utils.get_transaction_create_payload()
        transaction_create.tokenization_mode = TokenizationMode.FORCE_CREATION
        transaction_create.customers_presence = CustomersPresence.NOT_PRESENT
        transaction_create.completion_behavior = TransactionCompletionBehavior.COMPLETE_DEFERRED

        transaction = self.create(transaction_create)

        authorized_transaction = self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        self.assertEqual(
            TransactionState.AUTHORIZED,
            authorized_transaction.state,
            "Transaction state should be AUTHORIZED"
        )

        transaction_void = self.transactions_service.post_payment_transactions_id_void_online(
            authorized_transaction.id, SPACE_ID
        )

        voided_transaction = self.transactions_service.get_payment_transactions_id(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            TransactionVoidState.SUCCESSFUL,
            transaction_void.state,
            "Transaction void state should be SUCCESSFUL"
        )

        self.assertEqual(
            TransactionState.VOIDED,
            voided_transaction.state,
            "Transaction state should be VOIDED"
        )

        self.assertTrue(
            self.transactions_service.get_payment_transactions_id_check_token_creation_possible(
                voided_transaction.id, SPACE_ID
            ),
            "Should be not possible to create token for voided transaction"
        )

    def test_create_transaction_credentials_should_create_transaction_token(self):
        """
        Creates transaction token for a transaction.

        Verifies that:
        - Token contains space ID
        - Token contains transaction ID
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        credentials = self.get_credentials(transaction.id)

        self.assertTrue(
            credentials.startswith(str(SPACE_ID)),
            "Transaction credentials token should have valid format"
        )
        self.assertIsNotNone(transaction.id)
        self.assertTrue(
            str(transaction.id) in credentials,
            "Transaction credentials token should contain transaction id"
        )

    def test_fetch_iframe_url_should_return_valid_url(self):
        """
        Gets IFrame payment URL for transaction.

        Verifies that:
        - URL contains space ID
        - URL contains transaction ID
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        iframe_url = self.transactions_service.get_payment_transactions_id_iframe_javascript_url(
            transaction.id, SPACE_ID
        )

        self.assertTrue(
            str(SPACE_ID) in iframe_url,
            "IFrame JavaScript URL should contain space id"
        )

        self.assertTrue(
            str(transaction.id) in iframe_url,
            "IFrame JavaScript URL should contain transaction id"
        )

    def test_fetch_lightbox_url_should_return_valid_url(self):
        """
        Gets Lightbox payment URL for transaction.

        Verifies that:
        - URL contains space ID
        - URL contains transaction ID
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        lightbox_javascript_url = self.transactions_service.get_payment_transactions_id_lightbox_javascript_url(
            transaction.id, SPACE_ID
        )

        self.assertTrue(
            str(SPACE_ID) in lightbox_javascript_url,
            "Lightbox URL should contain space id"
        )

        self.assertTrue(
            str(transaction.id) in lightbox_javascript_url,
            "Lightbox URL should contain transaction id"
        )

    def test_fetch_invoice_should_return_pdf_file(self):
        """
        Creates, authorizes, completes transaction and gets invoice file.

        - Creates, authorizes and completes transaction
        - Gets transaction invoice file
        Verifies that:
            - File title contains "invoice"
            - File mime type is PDF
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        authorized_transaction = self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        fulfilled_transaction_completion = self.transactions_service.post_payment_transactions_id_complete_online(
            authorized_transaction.id, SPACE_ID
        )

        fulfilled_transaction = self.transactions_service.get_payment_transactions_id(
            transaction.id, SPACE_ID
        )

        invoice = self.transactions_service.get_payment_transactions_id_invoice_document(
            fulfilled_transaction.id, SPACE_ID
        )

        self.assertIsNotNone(invoice.title)
        self.assertTrue(
            "invoice" in invoice.title.lower(), "Invoice title should be present"
        )

        self.assertIsNotNone(invoice.mime_type)
        self.assertTrue(
            "pdf" in invoice.mime_type.lower(), "Invoice mimetype should be pdf"
        )

    def test_fetch_package_slip_should_return_pdf_file(self):
        """
        Creates, authorizes, completes transaction and gets packing slip.

        - Creates, authorizes and completes transaction
        - Gets transaction packing slip
        Verifies that:
            - File title contains "packing slip"
            - File mime type is PDF
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        authorized_transaction = self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        fulfilled_transaction_completion = self.transactions_service.post_payment_transactions_id_complete_online(
            authorized_transaction.id, SPACE_ID
        )

        fulfilled_transaction = self.transactions_service.get_payment_transactions_id(
            transaction.id, SPACE_ID
        )

        packing_slip = self.transactions_service.get_payment_transactions_id_packing_slip_document(
            fulfilled_transaction.id, SPACE_ID
        )

        self.assertIsNotNone(packing_slip.title)
        self.assertTrue(
            "packing slip" in packing_slip.title.lower(),
            "Packing slip title should be present"
        )

        self.assertIsNotNone(packing_slip.mime_type)
        self.assertTrue(
            "pdf" in packing_slip.mime_type.lower(),
            "Packing slip mimetype should be pdf"
        )

    def test_fetch_payment_methods_by_id_should_return_available_payment_methods(self):
        """
        Creates transaction and gets payment methods configuration.

        - Creates transaction
        - Gets payment methods configuration
        Verifies that:
            - Payment methods are present
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        method_configuration_list_response = self.transactions_service.get_payment_transactions_id_payment_method_configurations(
            transaction.id, self.INTEGRATION_MODE, SPACE_ID
        )

        self.assertIsNotNone(
            method_configuration_list_response.data, "The payment method list should be present"
        )
        self.assertFalse(
            len(method_configuration_list_response.data) == 0,
            "Payment methods should be configured for a given transaction in test space"
        )

    def test_fetch_transaction_with_credentials_should_return_transaction(self):
        """
        Creates transaction and finds it by credentials.

        - Creates transaction and gets its credentials
        - Finds transaction by credentials
        Verifies that:
            - Transaction is present
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        credentials = self.transactions_service.get_payment_transactions_id_credentials(
            transaction.id, SPACE_ID
        )

        retrieved_transaction = self.transactions_service.get_payment_transactions_by_credentials_credentials(
            credentials, SPACE_ID
        )

        self.assertIsNotNone(retrieved_transaction, "Transaction must be present")

    def test_fetch_payment_methods_with_credentials_should_return_available_payment_methods(self):
        """
        Creates transaction and gets payment methods configuration by credentials.

        - Creates transaction and gets its credentials
        - Gets payment methods configuration by credentials
        Verifies that:
            - Payment methods are present
        """

        transaction = self.create(Utils.get_transaction_create_payload())
        credentials = self.get_credentials(transaction.id)

        method_config_response = self.transactions_service.get_payment_transactions_by_credentials_credentials_payment_method_configurations(
            credentials, self.INTEGRATION_MODE, SPACE_ID
        )

        self.assertIsNotNone(
            method_config_response.data, "The payment method list should be present."
        )
        self.assertTrue(
            method_config_response.data,
            "Payment methods should be configured for a given transaction in test space"
        )

    def test_export_transactions_should_return_file(self):
        """
        Creates and exports a transaction.

        Verifies that:
            - Export file exists
        """

        transaction = self.create(Utils.get_transaction_create_payload())
        self.assertIsNotNone(transaction.id)

        fields = ["id"]
        export_data = self.transactions_service.get_payment_transactions_export(
            SPACE_ID, fields, 1, 0, order="id:ASC", query=f"id:{transaction.id}"
        )

        self.assertIsInstance(export_data, bytes)
        self.assertGreater(len(export_data), 0, "Exported data should be empty")

    def test_fetch_with_credentials_with_given_bad_credentials_should_fail(self):
        """
        Gets transaction by invalid credentials.

        - Attempts to retrieve a transaction using invalid credentials
        Verifies that:
        - Operation fails as expected
        """

        with self.assertRaises(ApiException, msg="Bad token should error response"):
            self.transactions_service.get_payment_transactions_by_credentials_credentials(
                "bad_credentials", SPACE_ID
            )

    def test_update_should_change_transaction_data(self):
        """
        Creates and updates a transaction.

        - Creates a new transaction
        - Updates it with new data
        Verifies that:
            - Update was successful
            - Version was incremented correctly
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        update = TransactionPending(version=2, language="en-GB")
        updated_transaction = self.transactions_service.patch_payment_transactions_id(
            transaction.id, SPACE_ID, update
        )

        self.assertEqual("en-GB", updated_transaction.language)
        self.assertEqual(
            transaction.merchant_reference,
            updated_transaction.merchant_reference,
            "Merchant reference should match."
        )
        self.assertEqual(2, updated_transaction.version, "Version should match")

    def test_process_one_click_token_and_redirect_with_credentials_should_return_payment_url(self):
        """
        Tests one-click token flow: creation, update, usage.

        - Creates token
        - Updates token settings (enabling one click payment)
        - Creates transaction with linked token
        - Processes and completes transaction
        - Creates new transaction and completes it using token
        """

        token_create = TokenCreate(externalId=str(uuid.uuid4()))
        token_create.state = CreationEntityState.ACTIVE
        token_create.token_reference = "testtoken"
        token_create.customer_id = TEST_CUSTOMER_ID
        token_create.customer_email_address = "test@domain.com"

        token = self.tokens_service.post_payment_tokens(SPACE_ID, token_create)

        self.assertIsNotNone(token.version)

        token_update = TokenUpdate(enabledForOneClickPayment=True, version=token.version)

        updated_token = self.tokens_service.patch_payment_tokens_id(token.id, SPACE_ID, token_update)

        transaction1 = self.tokens_service.post_payment_tokens_id_create_transaction_for_token_update(
            token.id, SPACE_ID
        )

        line_item = LineItemCreate(
            name="Red T-Shirt",
            unique_id="5412",
            type="PRODUCT",
            quantity=1,
            amount_including_tax=29.95,
            sku="red-t-shirt-789"
        )

        transaction1_pending = TransactionPending(
            lineItems=[line_item],
            version=2,
            currency="CHF"
        )

        self.transactions_service.patch_payment_transactions_id(
            transaction1.id, SPACE_ID, transaction1_pending
        )

        self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction1.id, SPACE_ID, MOCK_CARD_DATA
        )

        self.transactions_service.post_payment_transactions_id_complete_online(
            transaction1.id, SPACE_ID
        )

        transaction2_create = Utils.get_transaction_create_payload()
        transaction2_create.customer_id = TEST_CUSTOMER_ID
        transaction2_create.tokenization_mode = TokenizationMode.FORCE_CREATION_WITH_ONE_CLICK_PAYMENT
        transaction2_create.customers_presence = CustomersPresence.NOT_PRESENT
        transaction2_create.completion_behavior = TransactionCompletionBehavior.COMPLETE_DEFERRED

        transaction2 = self.create(transaction2_create)

        credentials1 = self.get_credentials(transaction1.id)
        credentials2 = self.get_credentials(transaction2.id)

        url = self.transactions_service.post_payment_transactions_by_credentials_credentials_one_click_tokens_id_process(
            credentials2, updated_token.id, SPACE_ID
        )

        self.assertIsNotNone(url, "Url should not be none")
        self.assertNotEqual(url, "", "Url should not be empty")

        read_transaction2 = self.transactions_service.get_payment_transactions_id(transaction2.id, SPACE_ID)

        self.assertNotEqual(read_transaction2.state, TransactionState.FAILED, "Transaction state should not be FAILED")
        self.assertNotEqual(read_transaction2.state, TransactionState.PENDING, "Transaction state should not be PENDING")

        completed_transaction1 = self.transactions_service.post_payment_transactions_id_complete_online(
            transaction1.id, SPACE_ID
        )

        self.assertEqual(completed_transaction1.state, TransactionCompletionState.SUCCESSFUL, "State must be SUCCESSFUL")

    def test_process_transaction_via_token_should_process_transaction(self):
        """
        Processes transaction via token

        - Creates token
        - Creates transaction with linked token
        - Processes and completes transaction
        - Creates new transaction and completes it using token
        """

        line_item = LineItemCreate(
            name="Red T-Shirt",
            unique_id="5412",
            type="PRODUCT",
            quantity=1,
            amount_including_tax=29.95,
            sku="red-t-shirt-789"
        )

        token_create = TokenCreate(externalId=str(uuid.uuid4()))
        token_create.state = CreationEntityState.ACTIVE
        token_create.token_reference = "testtoken"
        token_create.customer_email_address = TEST_CUSTOMER_ID
        token_create.customer_email_address = "test@domain.com"

        token = self.tokens_service.post_payment_tokens(SPACE_ID, token_create)

        transaction = self.tokens_service.post_payment_tokens_id_create_transaction_for_token_update(
            token.id, SPACE_ID
        )

        transaction1_pending = TransactionPending(
            lineItems = [line_item],
            version=2,
            currency="CHF"
        )

        self.transactions_service.patch_payment_transactions_id(
            transaction.id, SPACE_ID, transaction1_pending
        )

        self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        self.transactions_service.post_payment_transactions_id_complete_online(
            transaction.id, SPACE_ID
        )

        transaction2 = self.create(Utils.get_transaction_create_payload())

        transaction_pending = TransactionPending(
            token=token.id,
            version=transaction2.version,
            currency="CHF",
            lineItems=[line_item]
        )

        updated_transaction = self.transactions_service.patch_payment_transactions_id(
            transaction2.id, SPACE_ID, transaction_pending
        )

        charge = self.transactions_service.post_payment_transactions_id_process_with_token(
            updated_transaction.id, SPACE_ID
        )

        self.assertEqual(charge.state, TransactionCompletionState.SUCCESSFUL, "Charge state must be SUCCESSFUL")

        read_transaction = self.transactions_service.get_payment_transactions_id(transaction2.id, SPACE_ID)
        self.assertEqual(read_transaction.token.id, token.id, "Tokens id should match")
        self.assertEqual(read_transaction.state, TransactionState.FULFILL, "Transaction state must be FULFILLED")

    def test_process_without_user_interaction_should_process_transaction_properly(self):
        """
        Verifies non-interactive transaction processing.

        - Processes a transaction without user interaction
        Verifies that:
            - Transaction reaches the AUTHORIZED state
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        processed = self.transactions_service.post_payment_transactions_id_process_without_interaction(
            transaction.id, SPACE_ID
        )

        self.assertEqual(processed.id, transaction.id, "Transaction ids must match")
        self.assertEqual(processed.state, TransactionState.PROCESSING, "Transaction state should be PROCESSING")

    def test_fetch_one_click_token_should_return_response_without_exception(self):
        """
        Retrieves tokens by transaction credentials.

        - Creates a new transaction
        - Attempts to retrieve one-click tokens
        Verifies that:
            - Response data is present
        """

        transaction = self.create(Utils.get_transaction_create_payload())
        credentials = self.get_credentials(transaction.id)

        response = self.transactions_service.get_payment_transactions_by_credentials_credentials_one_click_tokens(
            credentials, SPACE_ID
        )

        self.assertIsNotNone(response)
        self.assertIsNotNone(response.data, "Token data should not be null")

    def test_process_transaction_with_three_d_secure_should_fulfill_transaction(self):
        """
        Processes transaction with 3-D security

        - Creates a new transaction
        - Processes transaction with 3-D security
        Verifies that:
            - Returned url contains space id
            - Returned url contains securityToken
            - Transaction state is fulfilled
        """

        transaction = self.create(Utils.get_transaction_create_payload())
        tokenized_card_data_create = TokenizedCardDataCreate(primaryAccountNumber=MOCK_CARD_DATA.card_data.primary_account_number)
        tokenized_card_data_create.expiry_date = MOCK_CARD_DATA.card_data.expiry_date

        tcr = TokenizedCardRequest(cardData=tokenized_card_data_create)
        tcr.payment_method_configuration = TEST_CARD_PAYMENT_METHOD_CONFIGURATION_ID

        url = self.transactions_service.post_payment_transactions_id_process_card_details_threed(
            transaction.id, SPACE_ID, tcr
        )

        processed = self.transactions_service.get_payment_transactions_id(transaction.id, SPACE_ID)

        self.assertIsNotNone(url, "Url should not be null")
        self.assertNotEqual(url, "", "Url should not be empty")
        self.assertNotEqual(processed.state, TransactionState.FAILED, "Transaction state should not be FAILED")
        self.assertNotEqual(processed.state, TransactionState.PENDING, "Transaction state should not be PENDING")

    def test_fetch_mobile_sdk_url_by_credentials_should_return_valid_url(self):
        """
        Gets mobile sdk url by credentials

        - Creates a new transaction
        - Gets mobile sdk url
        Verifies that:
            - Returned url contains space id
            - Returned url contains securityToken
        """

        transaction = self.create(Utils.get_transaction_create_payload())
        credentials = self.get_credentials(transaction.id)

        url = self.transactions_service.get_payment_transactions_by_credentials_credentials_mobile_sdk_url(
            credentials, SPACE_ID
        )

        self.assertIn(f"/s/{SPACE_ID}", url, "Space id should be present in url")
        self.assertIn("securityToken=", url, "Security token should be present in url")

    def test_fetch_terminal_receipts_should_return_valid_terminal_receipts_list(self):
        """
        Gets all terminal receipts for transaction.

        Verifies that:
            - List of terminal receipts is empty (as they were not created)
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        self.transactions_service.post_payment_transactions_id_complete_online(
            transaction.id, SPACE_ID
        )

        receipt_response = self.transactions_service.get_payment_transactions_id_terminal_receipts(
            transaction.id, TerminalReceiptFormat.TXT, 200, SPACE_ID
        )

        self.assertIsNotNone(receipt_response, "Response should be present")
        self.assertEqual(len(receipt_response.data), 0, "Response size should be 0 as no terminal receipts created for this transaction")

    def test_update_charge_flow_recipient_should_not_throw_exception(self):
        """
        Updates charge flow recipient for processing transaction

        Verifies that:
            - Operation was made without exceptions
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        self.transactions_service.post_payment_transactions_id_charge_flow_apply(
            transaction.id, SPACE_ID
        )

        try:
            self.transactions_service.post_payment_transactions_id_charge_flow_update_recipient(
                transaction.id, 1453447675844, "test2@domain.com", SPACE_ID
            )
        except Exception:
            self.fail("ApiException should not be thrown when updating recipient")

    def test_fetch_line_items_version_should_return_latest_line_items_version(self):
        """
        Gets last version of line items list

        - Creates a new transaction
        - Gets line items
        Verifies that:
            - Amount is correct
            - Version is correct
            - Transaction id is same in transaction and charge
        """

        transaction = self.create(Utils.get_transaction_create_payload())

        self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        self.transactions_service.post_payment_transactions_id_complete_online(
            transaction.id, SPACE_ID
        )

        line_items = self.transactions_service.get_payment_transactions_id_latest_line_item_version(
            transaction.id, SPACE_ID
        )

        self.assertEqual(line_items.version, 1, "Line items version should be 1")
        self.assertEqual(line_items.amount, 29.95, "Line items amount should be 29.95")
        self.assertEqual(line_items.linked_transaction, transaction.id, "Transaction ids should match")

    def create(self, transaction_create):
        return self.transactions_service.post_payment_transactions(SPACE_ID, transaction_create)

    def get_credentials(self, transaction_id):
        return self.transactions_service.get_payment_transactions_id_credentials(transaction_id, SPACE_ID)



# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from enum import Enum

class ApiExceptionErrorCodes(Enum):
    """
    This class represents possible HTTP error codes which can be sent by the API.
    This list corresponds to https://checkout.postfinance.ch/en-us/doc/api/web-service#_errors
    """
    # The request was not accepted often due to missing or invalid parameters
    BAD_REQUEST = 400
    # The necessary authentication credentials are missing or incorrect
    UNAUTHORIZED = 401
    # The application user is missing the required permissions
    FORBIDDEN = 403
    # The requested resource was not found
    NOT_FOUND = 404
    # The requested response format is not supported
    NOT_ACCEPTABLE = 406
    # The request conflicts with another request often because of to optimistic locking
    CONFLICT = 409
    # Too many operations in a bulk request
    PAYLOAD_TOO_LARGE = 413
    # The request provides unsupported or invalid data
    UNSUPPORTED_MEDIA_TYPE = 415
    # The pagination offset exceeds the limit
    RANGE_NOT_SATISFIABLE = 416
    # The request is well-formed but contains semantic errors
    UNPROCESSABLE_ENTITY = 422
    # Too many requests hit the API too quickly
    TOO_MANY_REQUESTS = 429
    # An internal error occurred on the server
    INTERNAL_SERVER_ERROR = 500

    def __init__(self, http_code):
        self._http_code = http_code

    @property
    def http_code(self):
        return self._http_code

    def matches(self, api_exception):
        """
        Checks if the given exception matches this error code.
        """
        return api_exception.status == self.http_code